/** \file Doc-Visu.h
    \brief Documentation detaillee de la partie visualisation de l'application : page associee au code. 
*/

/*! \page Doc-Visu  Documentation dtaille de la partie visualisation de l'application
 Cette page explique : 
 - \ref sec-visu1 (point de vue gnral)
 	- \ref subsection-visu12 : objet essentiel permettant de lier tous les objets de la scne
 	- \ref subsection-visu11 : application principale du module de visualisation
 	
 - \ref sec-visu2
 	
 - \ref sec-visu3
 	- \ref subsection-visu31
 	- \ref subsection-visu32
 
--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
   
 \section sec-visu1 Fonctionnement de la partie visualisation de l'animation 3D
 
 \subsection subsection-visu12 Le graphe de scne
 Le graphe de scne dfini par la classe \ref Scene permet de grer les diffrents objets de la scne :
 - La classe \ref Scene contient la liste des \ref Noeud de la scne (les enfants).
 - Chaque objet que nous souhaitons visualiser doit ainsi hriter de la classe \ref Noeud. Par exemple la classe \ref ObjetSimule hrite de la classe \ref Noeud.
 - Les diffrentes mthodes de \ref Scene vont permettre d'appeler les mthodes correspondantes pour chacun des objets \ref Noeud attachs  la scne. Ainsi l'appel de la mthode Simulation() de la classe Scene permet d'invoquer cette mthode pour l'ensemble des lments attachs  la scne.

 Les mthodes de la classe \ref Noeud doivent ainsi obligatoirement tre implmentes dans les classes relatives aux diffrents objets de la scne (mme si elles ne font rien). Cet appel est mis dans la fonction idle() qui assure galement le raffichage de la scne par la fonction glutPostRedisplay() aprs la mise  jour des positions des objets anims.
 
 
 \subsection subsection-visu11 Boucle GLUT

	L'application contient un objet \ref Scene correspondant au graphe de la scne qui contient tous les objets qui vont tre visualiss. Ces objets sont donc attachs  la scne ds leur cration. 
 
    La mthode init() permet d'initialiser les objets  partir des donnes fournies par les fichiers de donnes (cration du maillage, dfinition des positions des particules du maillage, etc.)  
 
 	La mthode display() permet l'affichage de la scne selon le type choisi (filaire, remplis, textur). C'est ainsi dans cette fonction qu'est invoque la mthode affiche() de chacun des objets de la \ref Scene de type \ref ObjetSimule.

 	La mise  jour des objets de la scne est effectue par l'appel de la mthode Simulation() de chacun des objets de la \ref Scene.
 
    Le bouton droit de la souris permet de faire apparatre le menu Glut de l'application : affichage plein cran, zoom, etc.
      
 \section sec-visu2 Reprsentation d'un objet de la scne
 
  Les principales structures de donnes relatives  un objet visualis sont les suivantes :
  	- _Interaction ( faire)
  	- _VIndices
  	- P
  	- _Normals
      
  La variable _Interaction permet de spcifier si il y a une interaction possible entre l'utilisateur et l'objet simul via le wand. La valeur de cette variable, "yes" ou "no", est fournie par le fichier de donnes mis en paramtre de l'excution. ( faire)
    
  La structure _VIndices contient les indices des sommets du maillage dcrivant l'objet visualis. Sa taille est dfinie par _VISize qui vaut trois fois le nombre de facettes puisqu'une facette est dfinie par trois sommets. Les trois premiers lments de cette structure correspondent aux indices des trois sommets dfinissant la premire facette de l'objet. L'ordre des indices des sommets au sein de cette structure suit ainsi l'ordre des facettes contenues dans la structure _vectFaceSet.
  
  La structure P contient les coordonnes de positions des sommets du maillage de l'objet. Sa taille correspond au nombre de particules. Le premier lment de cette structure (de type \ref Coord) correspond ainsi  la coordonne en x, y et z de la position du premier sommet de l'objet visualis. L'lment suivant correspond  la coordonne en x, y, et z du deuxime sommet, et ainsi de suite. Les donnes de cette structure sont mises  jour par la partie simulation de l'application. 
  
  La structure _Normals contient les coordonnes des normales des sommets de l'objet visualis. Sa taille est dfinie par _VSize qui est gale  trois fois le nombre de particules. Les trois premiers lments de _Normals correspondent aux coordonnes en x, y, z de la normale du premier sommet, et ainsi de suite. Les donnes de cette structure sont calcules  partir des coordonnes de positions des sommets (voir \ref subsection-visu32).
  
  
  \section sec-visu3 Algorithmes de la visualisation 
  Voici les principaux algorithmes implmentes au sein du module de visualisation :
    
  \subsection subsection-visu31 Calcul de la normale d'une face 
  Considrons une facette dfinie par les sommets A, B et C. La normale de cette facette est alors dfinie par 
  \f$\vec{n} = \frac{\vec{AB} \wedge \vec{AC}}{\| \vec{AB} \wedge \vec{AC} \|}\f$ 
  avec 
  \f$\vec{AB} = \left(
  \begin{array}{c}
  x_B - x_A\\
  y_B - y_A \\
  z_B - z_A
   \end{array}
   \right) \f$ 
   et \f$ \|\vec{u}\| = \sqrt{x^2 + y^2 + z^2} \f$ o x, y et z sont les coordonnes de \f$ \vec{u} \f$.
   
    
  \subsection subsection-visu32 Lissage des normales des sommets
  Afin d'viter d'avoir des "pics" entre les facettes, il faut lisser les normales des sommets. Voici les tapes de ce lissage :
  	- La premire tape consiste  initialiser  zro la normale en chacun des sommets du maillage.
  	- Ensuite pour chacune des facettes, nous allons 
  		- calculer sa normale (voir \ref subsection-visu31), 
  		- et ajouter cette valeur  la normale de chacun des sommets constituant la facette.
  	- Enfin la normale calcule pour chaque sommet est normalise en la divisant par sa norme. 
  	  
*/

