/** \file Bin2Text.cpp
  Conversion Binaire vers Texte des fichiers de donnees (points, masses, facettes et texture).
*/

#include <stdio.h>
#include <math.h>
#include <vector>
#include <fstream>
#include <iostream>

#include "Donnees.h"


/**
 * Initialisation des tableaux et construction du maillage a partir des donnees lues dans le fichier.
 */
int main(int argc, char** argv)
{  
  // Verification des arguments
  if (argc <5) 
    {
      std::cout << "Usage " << argv[0] << " fich_coord fich_facet fich_mass fich_text" << std::endl;	
      exit(1);
    }

  // Noms des fichiers de donnees binaires : fich_coord.bin fich_facet.bin fich_mass.bin fich_text.bin
  std::string n_fich_coord_bin = argv[1];
  std::string n_fich_facet_bin = argv[2];
  std::string n_fich_mass_bin = argv[3];
  std::string n_fich_text_bin = argv[4];

  // Noms des fichiers de donnees textes : fich_coord.bin.txt fich_facet.bin.txt fich_mass.bin.txt fich_text.bin.txt
  std::string n_fich_coord = n_fich_coord_bin + ".txt";
  std::string n_fich_facet = n_fich_facet_bin + ".txt";
  std::string n_fich_mass = n_fich_mass_bin + ".txt";
  std::string n_fich_text = n_fich_text_bin + ".txt";

  // Ecriture des fichiers textes
  std::ofstream fich_coord(n_fich_coord.c_str()) ;
  std::ofstream fich_facet(n_fich_facet.c_str()) ;
  std::ofstream fich_mass(n_fich_mass.c_str()) ;
  std::ofstream fich_text(n_fich_text.c_str()) ;

  // Lecture des fichiers binaires
  std::ifstream fich_coord_bin(n_fich_coord_bin.c_str()) ;
  std::ifstream fich_facet_bin(n_fich_facet_bin.c_str()) ;
  std::ifstream fich_mass_bin(n_fich_mass_bin.c_str()) ;
  std::ifstream fich_text_bin(n_fich_text_bin.c_str()) ;


  /** Ouverture des fichiers de donnees **/
  /* Fichier des points */
  // Fichier texte
  if ((fich_coord.eof()) || (! fich_coord)) 
    {
      std::cout << "Erreur d ouverture du fichier de donnees des points " 
		<< n_fich_coord << std::endl;
      exit(1);
    }

  // Fichier binaire
  if ((fich_coord_bin.eof()) || (! fich_coord_bin))
    {
      std::cout << "Erreur d ouverture du fichier binaire de donnees des points " 
		<< n_fich_coord_bin << std::endl;
      exit(1);
    }

  /* Fichier des faceSet */
  // Fichier texte
  if ((fich_facet.eof()) || (! fich_facet)) 
    {
      std::cout << "Erreur d ouverture du fichier de donnees des faceSet " 
		<< n_fich_facet << std::endl;
      exit(1);
    }
  
  // Fichier binaire
  if ((fich_facet_bin.eof()) || (! fich_facet_bin)) 
    {
      std::cout << "Erreur d ouverture du fichier binaire de donnees des faceSet " 
		<< n_fich_facet_bin << std::endl;
      exit(1);
    }

  /* Fichier des masses */
  // Fichier texte
  if ((fich_mass.eof()) || (! fich_mass)) 
    {
      std::cout << "Erreur d ouverture du fichier de donnees des masses " 
		<< n_fich_mass << std::endl;
      exit(1);
    }
  
  // Fichier binaire
  if ((fich_mass_bin.eof()) || (! fich_mass_bin)) 
    {
      std::cout << "Erreur d ouverture du fichier binaire de donnees des masses " 
		<< n_fich_mass_bin << std::endl;
      exit(1);
    }

  /* Fichier des textures */
  // Fichier texte
  if ((fich_text.eof()) || (! fich_text)) 
    {
      std::cout << "Erreur d ouverture du fichier de donnees des textures " 
		<< n_fich_text << std::endl;
      exit(1);
    }
  
  // Fichier binaire
  if ((fich_text_bin.eof()) || (! fich_text_bin)) 
    {
      std::cout << "Erreur d ouverture du fichier binaire de donnees des textures " 
		<< n_fich_text_bin << std::endl;
      exit(1);
    }


  /** Coordonnees des particules **/
  /* Nombre de particules */
  int nbparticules;
  fich_coord_bin.read( (char*)&nbparticules, sizeof(int) );
  Coord* allparticules = new Coord[nbparticules];
  fich_coord_bin.read( (char*)allparticules, sizeof(Coord)*nbparticules );
  fich_coord << nbparticules << std::endl;
  Coord coord;
  std::cout << "#particules=" << nbparticules << std::endl;

  /* Positions des particules */
  for (int i=0; i<nbparticules; ++i)
  {
    // Ecriture des valeurs 
    const Coord& coord = allparticules[i];
    fich_coord << coord.x << " " << coord.y << " " << coord.z << std::endl;

  }//for_i

  /* Fermeture des fichiers et liberation du tableau */
  fich_coord.close(); 
  fich_coord_bin.close(); 
  delete [] allparticules;
  

  /** Facettes du maillage **/
  /* Nombre de facettes du maillage */
  int nbfacets;
  fich_facet_bin.read( (char*)&nbfacets, sizeof(int) );
  Facet* allfacets = new Facet[nbfacets];
  fich_facet_bin.read( (char*)allfacets, sizeof(Facet)*nbfacets );
  std::cout << "#facets=" << nbfacets << std::endl;

  /* Facettes du maillage */
  for (int i=0; i<nbfacets; ++i)
    {
      // Ecriture des valeurs
      fich_facet << allfacets[i].fi << " "
                 << allfacets[i].fj << " "
                 << allfacets[i].fk << std::endl;
    }

  /* Fermeture des fichiers et liberation du tableau */
  fich_facet.close();
  fich_facet_bin.close();
  delete [] allfacets;


  /** Masses des particules **/
  /*  Nombre de masses */
  int nbmasss;
  fich_mass_bin.read( (char*)&nbmasss, sizeof(int) );
  Precision* allmasss = new Precision[nbmasss];
  fich_mass_bin.read( (char*)allmasss, sizeof(Precision)*nbmasss );
  std::cout << "#mass=" << nbmasss << std::endl;

  /* Masses des particules */
  for (int i=0; i<nbmasss; ++i)
    {
      // Ecriture des valeurs
      fich_mass << allmasss[i] << std::endl;
    }

  /* Fermeture des fichiers et liberation du tableau */
  fich_mass.close();
  fich_mass_bin.close();
  delete [] allmasss;


  /** Textures **/
  /* Nombre de coordonnes de la texture */
  int nbtext;
  fich_text_bin.read( (char*)&nbtext, sizeof(int) );
  Texture* alltext = new Texture[nbtext];
  fich_text_bin.read( (char*)alltext, sizeof(Texture)*nbtext );
  std::cout << "#text=" << nbtext << std::endl;

  /* Textures */
  for (int i=0; i<nbtext; ++i)
    {
      // Ecriture des valeurs
      fich_text << alltext[i].a << " "
		<< alltext[i].b << std::endl;
    }

  /* Fermeture des fichiers et liberation du tableau */
  fich_text.close();
  fich_text_bin.close();
  delete [] alltext;

	return 0;
}//main
