/*
 * Moteur.cpp : Animation d'une scene 3D
 * Copyright (C) 2007 Florence Zara, LIRIS
 *               florence.zara@liris.univ-lyon1.fr
 *               http://liris.cnrs.fr/florence.zara/
 *
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2.1 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


/** \file Moteur.cpp
*  \brief Animation 3D : programme principal avec boucle d animation GLUT. 
*/




/** Librairies de base **/
#include <sstream>
#include <string.h>
#include <iostream>

#ifdef WIN32
#include <string.h>
#else
#include <strings.h>
#endif



/** Pour GLUT **/
 /* Sous MAC */
#include <GLUT/glut.h>  

/* Sous Linux */
// #include <GL/glut.h> 

/** Trackball **/
#include "trackball.h"

/** Navigation dans fenetre glut **/
#include "Navigation.h"

/** Fichiers de l application **/
#include "Noeuds.h"
#include "Scene.h"
#include "Mesh.h"
#include "ObjetSimule.h"


/// Variables globales liees a la navigation dans la fenetre glut
int trackballBeginX, trackballBeginY;

GLuint trackballWidth, trackballHeight;
GLint trackballButton = GLUT_LEFT_BUTTON;
bool trackballTracking = false;

GLuint trackballLastTime;
float currentQuaternion[4];
float newQuaternion[4];

float scale_factor = 1.0;
GLint xGL, yGL;

eShading shadingMode = SHADING_SMOOTH;

bool normalDisplaying = false;



/// Graphe de scene 
Scene *S; 

/// Nombre d objets presents dans la scene
int _NbObj;

/// Choix de l affichage general : fil, remplissage ou texture
std::string _Affichage = "fil";

/// Compteur du temps de la boucle de la simulation 
int Tps = 0;	




/**
* function called whenever redisplay needed. 
 */
void display(void)		
{
	/* clear the display */
	glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );

	glPushMatrix();
	GLfloat m[4][4];
    build_rotmatrix(m, currentQuaternion);
    glMultMatrixf(&m[0][0]);
	glScalef(scale_factor, scale_factor, scale_factor);

	glDisable(GL_LIGHTING);
	// glDisable(GL_DEPTH_TEST);
	
	///----------- Affichage des 3 axes
    glLineWidth (2);
    glColor3f(1.0, 0.0, 0.0);
    glBegin(GL_LINES);
    glVertex3f(0.0,0.0,0.0);
    glVertex3f(1.0,0.0,0.0);
    glEnd();
	
    glLineWidth (2);
    glColor3f(0.0, 1.0, 0.0);
    glBegin(GL_LINES);
    glVertex3f(0.0,0.0,0.0);
    glVertex3f(0.0,1.0,0.0);
    glEnd();
	
    glLineWidth (2);
    glColor3f(0.0, 0.0, 1.0);
    glBegin(GL_LINES);
    glVertex3f(0.0,0.0,0.0);
    glVertex3f(0.0,0.0,1.0);
    glEnd();
	
	///------ Affichage des objets de la scene
	/** Choix de l affichage des faces **/
	// Affiche toutes les faces en mode ligne
	if (_Affichage == "fil")
		glPolygonMode(GL_FRONT_AND_BACK, GL_LINE);
	
	// Affiche les faces arrieres remplies
	else if (_Affichage == "remplissage")
		glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);
	
	// Plaquage de la texture 
	else if (_Affichage == "texture")
		glPolygonMode(GL_FRONT_AND_BACK, GL_FILL); 
	
	
	// Affichage des objets de la scene
	//	std::cout << "Affichage ...." << std::endl;
	S->affiche();
	
	glutSwapBuffers() ;
	glPopMatrix() ;
	glFlush() ;
		
}


/**
*
 */
void idle(void) 
{
	//std::cout << "Simulation continue ...." << std::endl;
	S->Simulation(Tps);
	Tps = Tps + 1;
	
	glutPostRedisplay();
}


/**
* Programme principal.
**/
int main(int argc, char *argv[])
{

	/**
	 -------------- Scene 3D -----------------
	 **/
			
		
	/// Tableau contenant les noms des fichiers de parametres des objets de la scene
	std::string *Fichier_Param;
	
	/// Fichier contenant les parametres de l execution 
	if (argc >= 2)
    {
		/* Nombre d objets a simuler */
		_NbObj = atoi(argv[1]);
		std::cout << "Nombre d objets dans la scene : " << _NbObj << std::endl;
		
		/* Tableau contenant les noms des fichiers de parametres */
		// Element 0 correspond au fichier contenant les parametres generaux de l animation
		// Element i correspond au fichier contenant les parametres de l objet i
		Fichier_Param = new std::string[_NbObj+1];
		
		// Recupere les noms des fichiers de donnees : 
		// arv[0] : executable, argv[1] : nb objet
		// Fichier_Param[0] de la simulation correspond a argv[2]
		for (int i=0; i<= _NbObj; i++)
			Fichier_Param[i] = argv[i+2]; 
	}
	
	else 
    {
		/* Usage de l execution du programme */
		std::cout << "Usage : ObjetDeformable NbObj <Fichier_Param_Anim> <Fichier_Param_Obj1> <Fichier_Param_Obj2> ..." << std::endl; 
		
		/* Arret du programme */
		exit(1);
		
    }
	
	/// Graphe de scene
	S = new Scene(Fichier_Param[0]);
	
	/// Creation des objets mailles de la scene 
	for (int i=1; i<= _NbObj; i++)
		S->attache(new ObjetSimule(Fichier_Param[i]));	  

		
	/// Initialisation des objets de la scene 
	// Initialisation pour l'animation
	S->init();
	

	/// Premier pas de la simulation 
	std::cout << "Simulation Begin...." << std::endl;
	S->Simulation(Tps);

		
	/**
		---------------------- Fenetre Glut ----------------------
		-------- Bouton droit de la souris pour voir le menu ------
	 **/
	int win;
	
	/* initialize GLUT system */
	glutInit(&argc, argv);		
	
	glutInitDisplayMode(GLUT_RGB | GLUT_DOUBLE);
	
	/* width=400pixels height=500pixels */
	glutInitWindowSize(400,500);			
	
	/* create window */
	win = glutCreateWindow("Animation 3D");	
	
	/* from this point on the current window is win */
	/* set background to black */
	glClearColor(0.0,0.0,0.0,0.0);	
	
	/* how object is mapped to window */
	gluOrtho2D(0,400,0,500);		
	
	/* set window's display callback */
	glutDisplayFunc(display);	
		
	/* set window's key callback */
	glutKeyboardFunc(keyboard);
	glutReshapeFunc(reshape);
	glutMotionFunc(motion);
	glutMouseFunc(mouse);
	glutVisibilityFunc(visible);
	
	glutCreateMenu(menu);
	glutAddMenuEntry("Menu", 0);
	glutAddMenuEntry("", 0);
	glutAddMenuEntry("[f]   Toggle fullscreen on/off", 'f');
	glutAddMenuEntry("[z]   Zoom out", 'z');
	glutAddMenuEntry("[Z]   Zoom in", 'Z');
	glutAddMenuEntry("[s]   switch shading mode", 's');
	glutAddMenuEntry("[n]   normal displaying", 'n');
	glutAddMenuEntry("[Esc] Quit", 27);
	glutAttachMenu(GLUT_RIGHT_BUTTON);
	
	trackball(currentQuaternion, 0.0, 0.0, 0.0, 0.0);
	
    /* start processing events... */
	glutMainLoop();			
	
	/* execution never reaches this point */
	return 0;
}