/*
 * Navigation.cpp : fonctions associees a la fenetre glut
 * Copyright (C) 2007 Sylvain Brandel, LIRIS
 *               sylvain.brandel@liris.univ-lyon1.fr
 *               http://liris.cnrs.fr/sylvain.brandel
 *
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2.1 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


/** \file Navigation.cpp
\brief fonctions associees a la fenetre glut
*/

#include "Navigation.h"
#include "trackball.h"

#include <iostream>

/**
* Gestion de la taille de la fenetre Glut.
 */
 void reshape(int width, int height) 
{
	// cout << "-- reshape (" << width << "," << height << ")" << endl;
	
	xGL = width;
	yGL = height;
	
	glViewport(0, 0, xGL, yGL);
	
	glMatrixMode (GL_PROJECTION);
	glLoadIdentity ();
	
	float ratio = (float)width/(float)height;
	glOrtho(-2, 2, -2 / ratio, 2 / ratio, -10.0, 10.0);
	
	glMatrixMode (GL_MODELVIEW);
	glLoadIdentity ();
	// gluLookAt(2, 1, 2, 0, 0, 0, 0, 1, 0);
	
	GLfloat mat_diffuse[] = { 0.5, 0.5, 0.9, 1.0 };
	GLfloat mat_specular[] = { 1.0, 1.0, 1.0, 1.0 };
	GLfloat mat_shininess[] = { 100.0 };
	GLfloat light_position[] = { 10.0, 10.0, 30.0, 1.0 };
	
	glMaterialfv(GL_FRONT, GL_DIFFUSE, mat_diffuse);
	glMaterialfv(GL_FRONT, GL_SPECULAR, mat_specular);
	glMaterialfv(GL_FRONT, GL_SHININESS, mat_shininess);
	
	glLightfv(GL_LIGHT0, GL_POSITION, light_position);
	
	glEnable(GL_LIGHTING);
	glEnable(GL_LIGHT0);
	glDepthFunc(GL_LEQUAL);
	glEnable(GL_DEPTH_TEST);
	glEnable(GL_AUTO_NORMAL);
	glEnable(GL_NORMALIZE);
	
	trackballWidth  = width;
	trackballHeight = height;
}


/**
* Mouvement.
 */
void motion(int x, int y) 
{
	if (trackballTracking) 
	{
		trackball(newQuaternion,
				  (2.0 * trackballBeginX - trackballWidth) / trackballWidth,
				  (trackballHeight - 2.0 * trackballBeginY) / trackballHeight,
				  (2.0 * x - trackballWidth) / trackballWidth,
				  (trackballHeight - 2.0 * y) / trackballHeight);
		add_quats(newQuaternion, currentQuaternion, currentQuaternion);
		
		trackballLastTime = glutGet(GLUT_ELAPSED_TIME);
		
		trackballBeginX = x;
		trackballBeginY = y;
	}
	
	glutPostRedisplay();
}


/**
* Gestion de la souris.
 */
void mouse(int button, int state, int x, int y) 
{
	std::cout << "Position de la souris : " << x << " " << y << std::endl;
	
	if (state == GLUT_DOWN && button == trackballButton) 
	{
		
	//	glutIdleFunc(0);
		
		trackballTracking = true;
		
		trackballLastTime = glutGet(GLUT_ELAPSED_TIME);
		
		trackballBeginX = x;
		trackballBeginY = y;
	}
	else {
		if (state == GLUT_UP && button == trackballButton) {
			trackballTracking = false;
		}
	}
	
	glutPostRedisplay();
}


/**
* Gestion du clavier.
 */
void keyboard(unsigned char key, int x, int y) 
{
	static int fullscreen = 0;
	static int old_x = 50;
	static int old_y = 50;
	static int old_width = 512;
	static int old_height = 512;
	
	switch (key) {
		case 27:
			exit(0);
			break;
			
		case 'f':
			fullscreen = !fullscreen;
			if (fullscreen) {
				old_x = glutGet(GLUT_WINDOW_X);
				old_y = glutGet(GLUT_WINDOW_Y);
				old_width = glutGet(GLUT_WINDOW_WIDTH);
				old_height = glutGet(GLUT_WINDOW_HEIGHT);
				glutFullScreen();
			}
				else {
					glutReshapeWindow(old_width, old_height);
					glutPositionWindow(old_x, old_y);
				}
				break;
			
		case 'z':
			if (scale_factor > 0)
				scale_factor -= 0.05;
			break;
			
		case 'Z':
			scale_factor += 0.05;
			break;
			
		case 's':
			if (shadingMode == SHADING_WIRED)     { shadingMode = SHADING_SMOOTH; break; }
			if (shadingMode == SHADING_SMOOTH)    { shadingMode = SHADING_NO_SMOOTH; break; }
			if (shadingMode == SHADING_NO_SMOOTH) { shadingMode = SHADING_WIRED; break; }
			
		case 'n':
			normalDisplaying = ! normalDisplaying;
			break;
	}
	
	glutPostRedisplay();
}


/**
* Menu.
 */
void menu(int item) 
{
	keyboard((unsigned char)item, 0, 0);
}


/**
*
 */
void visible(int state) 
{
	if (state == GLUT_VISIBLE) {
		 glutIdleFunc(idle);
		//glutIdleFunc(NULL);
	}
	else
		//glutIdleFunc(NULL);
		glutIdleFunc(idle);
	
}
